﻿namespace TabsStudioMarker
{
    class ContextMenuHelper
    {
        public ContextMenuHelper(CommandsImpl commandsImpl_, Model model_)
        {
            commandsImpl = commandsImpl_;
            model = model_;

            highlightMenuItem = new System.Windows.Controls.MenuItem();
            System.Windows.Controls.StackPanel markPanel = new System.Windows.Controls.StackPanel();
            markPanel.Orientation = System.Windows.Controls.Orientation.Horizontal;
            {
                System.Windows.Controls.Label mark = new System.Windows.Controls.Label();
                mark.Padding = new System.Windows.Thickness(0);
                mark.Margin = new System.Windows.Thickness(0,0,5,0);
                mark.VerticalAlignment = System.Windows.VerticalAlignment.Center;
                mark.Content = "Mark";
                markPanel.Children.Add(mark);
            }
            for (uint i = 1; i <= 5; ++i)
            {
                System.Windows.Controls.Button button = new System.Windows.Controls.Button();
                {
                    System.Windows.Shapes.Rectangle icon = new System.Windows.Shapes.Rectangle();
                    icon.Width = 12;
                    icon.Height = 16;
                    icon.Fill = new System.Windows.Media.SolidColorBrush(indexColor(i));
                    button.Content = icon;
                    markPanel.Children.Add(button);
                }
                uint ivalueForLambda = i;
                button.Click += (sender, e) => OnIndexHighlight(ivalueForLambda);
            }
            highlightMenuItem.Header = markPanel;
            highlightMenuItem.Click += (sender, e) => OnIndexHighlight(1);

            removeHighlightingMenuItem = new System.Windows.Controls.MenuItem();
            removeHighlightingMenuItem.Header = "Remove highlighting";
            removeHighlightingMenuItem.Click += OnRemoveHighlighting;

            removeAllHighlightingMenuItem = new System.Windows.Controls.MenuItem();
            removeAllHighlightingMenuItem.Header = "Remove all highlighting";
            removeAllHighlightingMenuItem.Click += OnRemoveAllHighlighting;

            closeAllButHighlightedMenuItem = new System.Windows.Controls.MenuItem();
            closeAllButHighlightedMenuItem.Header = "Close all but highlighted";
            closeAllButHighlightedMenuItem.Click += OnCloseAllButHighlighted;
        }

        public void StoreTabs(TabsStudioExt.Tab tab, TabsStudioExt.Tabs tabs)
        {
            storedTab = tab;
            storedTabs = tabs;
        }

        public void PlaceMenuItem(System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    // Only add our menu items when SetColor is present.
                    if ((item as System.Windows.Controls.Control).Name.Equals("SetColor"))
                    {
                        menu.Items.Insert(TabsStudioSeparatorIndex(menu, item), new System.Windows.Controls.Separator());
                        menu.Items.Insert(TabsStudioSeparatorIndex(menu, item), highlightMenuItem);
                        if (model.GetHighlightIndex(storedTab.TabItem) > 0)
                            menu.Items.Insert(TabsStudioSeparatorIndex(menu, item), removeHighlightingMenuItem);
                        menu.Items.Insert(TabsStudioSeparatorIndex(menu, item), removeAllHighlightingMenuItem);
                        menu.Items.Insert(TabsStudioSeparatorIndex(menu, item), closeAllButHighlightedMenuItem);
                        storedMenu = menu;
                        break;
                    }
                }
            }
        }

        private int TabsStudioSeparatorIndex(System.Windows.Controls.ContextMenu menu, object setColorItem)
        {
            return menu.Items.IndexOf(setColorItem) - 1;
        }

        private System.Windows.Media.Color indexColor(uint i)
        {
            string result = null;
            switch (i)
            {
                case 1:
                    result = "#FF0000";
                    break;
                case 2:
                    result = "#00AE00";
                    break;
                case 3:
                    result = "#009CFF";
                    break;
                case 4:
                    result = "#BC00FF";
                    break;
                case 5:
                    result = "#000000";
                    break;
            }
            return (System.Windows.Media.Color)System.Windows.Media.ColorConverter.ConvertFromString(result);
        }

        private void OnIndexHighlight(uint index)
        {
            model.SetHighlightIndex(storedTab.TabItem, index);
            storedMenu.IsOpen = false;
        }

        private void OnRemoveHighlighting(object sender, System.Windows.RoutedEventArgs e)
        {
            model.SetHighlightIndex(storedTab.TabItem, 0);
        }

        private void OnRemoveAllHighlighting(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.RemoveAllHighlighting();
        }

        private void OnCloseAllButHighlighted(object sender, System.Windows.RoutedEventArgs e)
        {
            commandsImpl.CloseAllButHighlighted(storedTabs);
        }

        private Model model;
        private System.Windows.Controls.MenuItem highlightMenuItem, removeHighlightingMenuItem, removeAllHighlightingMenuItem;
        private System.Windows.Controls.MenuItem closeAllButHighlightedMenuItem;
        private TabsStudioExt.Tab storedTab;
        private TabsStudioExt.Tabs storedTabs;
        private CommandsImpl commandsImpl;
        private System.Windows.Controls.ContextMenu storedMenu;
    }
}
