﻿namespace TabsStudioMarker
{
    public class Marker : TabsStudioExt.ITabsStudioAddin, EnvDTE.IDTCommandTarget, TabsStudioExt.IStyler, TabsStudioExt.IConfigurable
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;

            storage = new OptionsStorage(engine.GetAddinSettingsDirectory());
            options = storage.Load();
            
            model = new Model(engine, dte, options);

            engine.Initialize += OnInitialize;
            engine.TabCreated += OnTabCreated;
            engine.TabDestroyed += OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;

            moveStateSaver = new MoveStateSaver(engine, model);
            solutionStateSaver = new SolutionStateSaver(dte, model);

            solutionEventsSink = new SolutionEventsSink(solutionStateSaver);
            System.IServiceProvider serviceProvider = new Microsoft.VisualStudio.Shell.ServiceProvider(
                    dte as Microsoft.VisualStudio.OLE.Interop.IServiceProvider);
            vsSolution = serviceProvider.GetService(typeof(Microsoft.VisualStudio.Shell.Interop.SVsSolution)) as
                Microsoft.VisualStudio.Shell.Interop.IVsSolution;
            vsSolution.AdviseSolutionEvents(solutionEventsSink, out sinkCookie);

            buildEvents = new BuildEventsListener(dte, solutionStateSaver);
            commandEvents = new CommandEventsListener(dte, solutionStateSaver);
            commandsImpl = new CommandsImpl(engine, dte, model);
            commands = new Commands(engine, dte, commandsImpl);
            contextMenuHelper = new ContextMenuHelper(commandsImpl, model);
        }

        public void OnDisconnection()
        {
            vsSolution.UnadviseSolutionEvents(sinkCookie);

            engine.Initialize -= OnInitialize;
            engine.TabCreated -= OnTabCreated;
            engine.TabDestroyed -= OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;

            moveStateSaver.Close();
            model.Close();
            buildEvents.Close();
            commandEvents.Close();
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioMarker=\"clr-namespace:TabsStudioMarker;assembly=Marker\"";
        }

        public void QueryStatus(string CmdName, EnvDTE.vsCommandStatusTextWanted NeededText,
                ref EnvDTE.vsCommandStatus StatusOption, ref System.Object CommandText)
        {
            commands.QueryStatus(CmdName, NeededText, ref StatusOption, ref CommandText);
        }

        public void Exec(string CmdName, EnvDTE.vsCommandExecOption ExecuteOption, ref System.Object VariantIn,
            ref System.Object VariantOut, ref bool Handled)
        {
            commands.Exec(CmdName, ExecuteOption, ref VariantIn, ref VariantOut, ref Handled);
        }

        public void Configure(System.Windows.Window parent)
        {
            OptionsDialog dialog = new OptionsDialog(options);
            dialog.Owner = parent;
            dialog.WindowStartupLocation = System.Windows.WindowStartupLocation.CenterOwner;
            if (dialog.ShowDialog().GetValueOrDefault())
                storage.Save(options);
        }

        private void OnInitialize(object sender, System.EventArgs e)
        {
            UpdatePresentationStyle();
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            e.Tab.TabItem.MouseUp += OnClick;
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            e.Tab.TabItem.MouseUp -= OnClick;            
        }

        private void OpeningTabExtensionContextMenu(object sender, TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            contextMenuHelper.StoreTabs(e.Tab, e.Tabs);
            contextMenuHelper.PlaceMenuItem(e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            contextMenuHelper.StoreTabs(e.Tab, e.Tabs);
            contextMenuHelper.PlaceMenuItem(e.ContextMenu);
        }

        private void OnClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if ((System.Windows.Forms.Control.ModifierKeys & System.Windows.Forms.Keys.Control) != 0)
                commandsImpl.ToggleTabHighlighting(sender as System.Windows.Controls.TabItem);
        }

        private void UpdatePresentationStyle()
        {
            TabsStudioExt.Presentation presentation = new TabsStudioExt.Presentation("", "");
            {
                TabsStudioExt.PresentationStyle style = new TabsStudioExt.PresentationStyle();
                style.Triggers.Add(LoadString("TabInternalsHighlighting.xml"));
                presentation.Styles.Add("TabsStudio:TabInternals", style);
            }
            engine.UpdatePresentationStyles("Marker", presentation);
        }

        private static string LoadString(string resource)
        {
            using (System.IO.Stream stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                "Marker." + resource))
            {
                using (System.IO.StreamReader reader = new System.IO.StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
 
        private OptionsStorage storage;
        private Options options;

        private Model model;
        private MoveStateSaver moveStateSaver;
         
        private Microsoft.VisualStudio.Shell.Interop.IVsSolution vsSolution;
        private SolutionStateSaver solutionStateSaver;
        private SolutionEventsSink solutionEventsSink;
        private uint sinkCookie;

        private BuildEventsListener buildEvents;
        private CommandEventsListener commandEvents;
        private CommandsImpl commandsImpl;
        private Commands commands;
        private ContextMenuHelper contextMenuHelper;
    }
}