﻿namespace TabsStudioMarker
{
    class MoveStateSaver
    {
        public MoveStateSaver(TabsStudioExt.ITabsStudioEngine engine_, Model model_)
        {
            engine = engine_;
            model = model_;
            lastCreatedWindows = new System.Collections.Generic.List<EnvDTE.Window>();
            lastDestroyedWindows = new System.Collections.Generic.List<EnvDTE.Window>();

            engine.TabExtensionCreated += OnExtensionCreated;
            engine.TabExtensionDestroyed += OnExtensionDestroyed;
        }

        public void Close()
        {
            engine.TabExtensionCreated -= OnExtensionCreated;
            engine.TabExtensionDestroyed -= OnExtensionDestroyed;
        }

        private void OnExtensionCreated(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabItem != lastCreatedTab)
            {
                lastCreatedTab = e.Tab.TabItem;
                lastCreatedWindows.Clear();
            }
            lastCreatedWindows.Add(e.TabExtension.Window);

            if (lastDestroyedWindows.Contains(e.TabExtension.Window))
            {
                uint colorIndex = model.GetHighlightIndex(lastDestroyedTab);
                if (colorIndex > 0)
                    model.SetHighlightIndex(lastCreatedTab, colorIndex);
            }
        }

        private void OnExtensionDestroyed(object sender, TabsStudioExt.TabExtensionEventArgs e)
        {
            if (e.Tab.TabItem != lastDestroyedTab)
            {
                lastDestroyedTab = e.Tab.TabItem;
                lastDestroyedWindows.Clear();
            }
            lastDestroyedWindows.Add(e.TabExtension.Window);

            if (lastCreatedWindows.Contains(e.TabExtension.Window))
            {
                uint colorIndex = model.GetHighlightIndex(lastDestroyedTab);
                if (colorIndex > 0)
                    model.SetHighlightIndex(lastCreatedTab, colorIndex);
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private Model model;
        private System.Windows.Controls.TabItem lastCreatedTab, lastDestroyedTab;
        private System.Collections.Generic.List<EnvDTE.Window> lastCreatedWindows, lastDestroyedWindows;
    }
}
